import threading
from time import time

from starlette.middleware.base import (BaseHTTPMiddleware,
                                       RequestResponseEndpoint)
from starlette.requests import Request
from starlette.responses import Response
from starlette.types import ASGIApp

from api_analytics.core import log_request


class Analytics(BaseHTTPMiddleware):
    def __init__(self, app: ASGIApp, api_key: str) -> None:
        super().__init__(app)
        self.api_key = api_key

    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
        start = time()
        response: Response = await call_next(request)
        elapsed = time() - start
        
        json = {
            'api_key': self.api_key,
            'hostname': request.url.hostname,
            'path': request.url.path,
            'user_agent': request.headers['user-agent'],
            'method': request.method,
            'status': int(response.status_code),
            'response_time': int(elapsed * 1000),
            'framework': 'FastAPI'
        }
        threading.Thread(target=log_request, args=(json,)).start()
        return response



import requests

method_map = {
    'GET': 0,
    'POST': 1,
    'PUT': 2,
    'PATCH': 3,
    'DELETE': 4,
    'OPTIONS': 5,
    'CONNECT': 6,
    'HEAD': 7,
    'TRACE': 8,
}

framework_map = {
    'FastAPI': 0,
    'Flask': 1,
}

def log_request(json: dict):
    json['method'] = method_map[json['method']]
    json['framework'] = framework_map[json['framework']]
    requests.post('https://api-analytics-server.vercel.app/api/log-request', json=json)



from setuptools import setup

long_description = open("README.md").read()

setup(
    name="fastapi-analytics",
    version="1.0.1",
    author="Tom Draper",
    author_email="tomjdraper1@gmail.com",
    license="MIT",
    description="Monitoring and analytics for FastAPI.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tom-draper/api-analytics",
    key_words="analytics api dashboard fastapi",
    install_requires=['fastapi'],
    packages=["api_analytics"],
    python_requires=">=3.6",
)